import { Component, Input, Output, Inject, EventEmitter, Host, OnInit } from 'ng-metadata/core';
import { CorTabPanelComponent } from './cor-tab-panel.component';


/**
 * A component that creates a single tab under a cor-tabs component.
 */
@Component({
  selector: 'cor-tab',
  templateUrl: '/static/js/directives/ui/cor-tabs/cor-tab.component.html',
  legacy: {
    transclude: true,
  }
})
export class CorTabComponent implements OnInit {
  @Input('@') public tabId: string;
  @Input('@') public tabTitle: string;

  @Output() public tabInit: EventEmitter<any> = new EventEmitter();
  @Output() public tabShown: EventEmitter<any> = new EventEmitter();
  @Output() public tabHidden: EventEmitter<any> = new EventEmitter();

  // Whether this is the active tab.
  private isActive: boolean = false;

  constructor(@Host() @Inject(CorTabPanelComponent) private parent: CorTabPanelComponent) {
  }

  public ngOnInit(): void {
    this.parent.addTab(this);
  }

  public changeState(isActive: boolean): void {
    this.isActive = isActive;
    if (isActive) {
      this.tabInit.emit({});
      this.tabShown.emit({});
    } else {
      this.tabHidden.emit({});
    }
  }

  private tabClicked(): void {
    this.parent.tabClicked(this);
  }
}
